@@ -19,19 +19,20 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -52,7 +53,6 @@ typedef enum {
 #define DEFAULT_BORDER 0.25
 
 typedef struct _Pgram Pgram;
-typedef struct _PgramDefaultsDialog PgramDefaultsDialog;
 
 struct _Pgram {
   Element element;
@@ -67,67 +67,48 @@ struct _Pgram {
   real shear_angle, shear_grad;
 
   Text *text;
+  TextAttributes attrs;
   real padding;
 };
 
 typedef struct _PgramProperties {
-  Color *fg_color;
-  Color *bg_color;
   gboolean show_background;
-  real border_width;
   real shear_angle;
-
   real padding;
-  Color *font_color;
 } PgramProperties;
 
-struct _PgramDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-  GtkSpinButton *shear_angle;
-
-  GtkSpinButton *padding;
-  DiaFontSelector *font;
-  GtkSpinButton *font_size;
-};
-
-
-static PgramDefaultsDialog *pgram_defaults_dialog;
 static PgramProperties default_properties;
 
 static real pgram_distance_from(Pgram *pgram, Point *point);
 static void pgram_select(Pgram *pgram, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void pgram_move_handle(Pgram *pgram, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* pgram_move_handle(Pgram *pgram, Handle *handle,
+				       Point *to, ConnectionPoint *cp,
+				       HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void pgram_move(Pgram *pgram, Point *to);
-static void pgram_draw(Pgram *pgram, Renderer *renderer);
+static ObjectChange* pgram_move(Pgram *pgram, Point *to);
+static void pgram_draw(Pgram *pgram, DiaRenderer *renderer);
 static void pgram_update_data(Pgram *pgram, AnchorShape h, AnchorShape v);
 static Object *pgram_create(Point *startpoint,
 			  void *user_data,
 			  Handle **handle1,
 			  Handle **handle2);
 static void pgram_destroy(Pgram *pgram);
-static Object *pgram_copy(Pgram *pgram);
 
 static PropDescription *pgram_describe_props(Pgram *pgram);
-static void pgram_get_props(Pgram *pgram, Property *props, guint nprops);
-static void pgram_set_props(Pgram *pgram, Property *props, guint nprops);
+static void pgram_get_props(Pgram *pgram, GPtrArray *props);
+static void pgram_set_props(Pgram *pgram, GPtrArray *props);
 
 static void pgram_save(Pgram *pgram, ObjectNode obj_node, const char *filename);
 static Object *pgram_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *pgram_get_defaults(void);
-static void pgram_apply_defaults(void);
 
 static ObjectTypeOps pgram_type_ops =
 {
   (CreateFunc) pgram_create,
   (LoadFunc)   pgram_load,
   (SaveFunc)   pgram_save,
-  (GetDefaultsFunc)   pgram_get_defaults,
-  (ApplyDefaultsFunc) pgram_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType pgram_type =
@@ -144,7 +125,7 @@ static ObjectOps pgram_ops = {
   (DrawFunc)            pgram_draw,
   (DistanceFunc)        pgram_distance_from,
   (SelectFunc)          pgram_select,
-  (CopyFunc)            pgram_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            pgram_move,
   (MoveHandleFunc)      pgram_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -172,7 +153,8 @@ static PropDescription pgram_props[] = {
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
-  PROP_STD_TEXT,
+  PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_SAVED_TEXT,
   
   { NULL, 0, 0, NULL, NULL, NULL, 0}
 };
@@ -195,89 +177,31 @@ static PropOffset pgram_offsets[] = {
     offsetof(Pgram, line_style), offsetof(Pgram, dashlength) },
   { "shear_angle", PROP_TYPE_REAL, offsetof(Pgram, shear_angle) },
   { "padding", PROP_TYPE_REAL, offsetof(Pgram, padding) },
+  {"text",PROP_TYPE_TEXT,offsetof(Pgram,text)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Pgram,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Pgram,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Pgram,attrs.color)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Pgram,attrs.alignment)},
   { NULL, 0, 0 },
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "text_font" },
-  { "text_height" },
-  { "text_colour" },
-  { "text" }
-};
-
 static void
-pgram_get_props(Pgram *pgram, Property *props, guint nprops)
+pgram_get_props(Pgram *pgram, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&pgram->element.object, 
-                                    pgram_offsets, props, nprops))
-    return;
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < 4; i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_FONT;
-      PROP_VALUE_FONT(props[i]) = pgram->text->font;
-    } else if (pquark == quarks[1].q) {
-      props[i].type = PROP_TYPE_REAL;
-      PROP_VALUE_REAL(props[i]) = pgram->text->height;
-    } else if (pquark == quarks[2].q) {
-      props[i].type = PROP_TYPE_COLOUR;
-      PROP_VALUE_COLOUR(props[i]) = pgram->text->color;
-    } else if (pquark == quarks[3].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      PROP_VALUE_STRING(props[i]) = text_get_string_copy(pgram->text);
-    }
-  }
+  text_get_attributes(pgram->text,&pgram->attrs);
+  object_get_props_from_offsets(&pgram->element.object,
+                                pgram_offsets,props);
 }
 
 static void
-pgram_set_props(Pgram *pgram, Property *props, guint nprops)
+pgram_set_props(Pgram *pgram, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&pgram->element.object, 
-                                     pgram_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < 4; i++)
-        quarks[i].q = g_quark_from_static_string(quarks[i].name);
-
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
-        text_set_font(pgram->text, PROP_VALUE_FONT(props[i]));
-      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
-        text_set_height(pgram->text, PROP_VALUE_REAL(props[i]));
-      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
-        text_set_color(pgram->text, &PROP_VALUE_COLOUR(props[i]));
-      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
-        text_set_string(pgram->text, PROP_VALUE_STRING(props[i]));
-      }
-    }
-  }
-  pgram->shear_grad = tan(M_PI/2.0 - M_PI/180.0 * pgram->shear_angle);
+  object_set_props_from_offsets(&pgram->element.object,
+                                pgram_offsets,props);
+  apply_textattr_properties(props,pgram->text,"text",&pgram->attrs);
   pgram_update_data(pgram, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 }
 
-static void
-pgram_apply_defaults()
-{
-  default_properties.shear_angle = M_PI/180.0 * gtk_spin_button_get_value_as_float(pgram_defaults_dialog->shear_angle);
-  default_properties.show_background = gtk_toggle_button_get_active(pgram_defaults_dialog->show_background);
-
-  default_properties.padding = gtk_spin_button_get_value_as_float(pgram_defaults_dialog->padding);
-  attributes_set_default_font(
-	dia_font_selector_get_font(pgram_defaults_dialog->font),
-	gtk_spin_button_get_value_as_float(pgram_defaults_dialog->font_size));
-}
-
 static void
 init_default_values() {
   static int defaults_initialized = 0;
@@ -290,112 +214,6 @@ init_default_values() {
   }
 }
 
-static GtkWidget *
-pgram_get_defaults()
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *checkpgram;
-  GtkWidget *shear_angle;
-  GtkWidget *padding;
-  GtkWidget *fontsel;
-  GtkWidget *font_size;
-  GtkAdjustment *adj;
-  Font *font;
-  real font_height;
-
-  if (pgram_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    pgram_defaults_dialog = g_new(PgramDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    pgram_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkpgram = gtk_check_button_new_with_label(_("Draw background"));
-    pgram_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkpgram );
-    gtk_widget_show(checkpgram);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkpgram, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Shear angle:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(60.0, 45.0, 135.0, 1.0,10.0,10.0);
-    shear_angle = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(shear_angle), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(shear_angle), TRUE);
-    pgram_defaults_dialog->shear_angle = GTK_SPIN_BUTTON(shear_angle);
-    gtk_box_pack_start(GTK_BOX (hbox), shear_angle, TRUE, TRUE, 0);
-    gtk_widget_show (shear_angle);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Text padding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
-    padding = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
-    pgram_defaults_dialog->padding = GTK_SPIN_BUTTON(padding);
-    gtk_box_pack_start(GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-    gtk_widget_show (padding);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    fontsel = dia_font_selector_new();
-    pgram_defaults_dialog->font = DIAFONTSELECTOR(fontsel);
-    gtk_box_pack_start (GTK_BOX (hbox), fontsel, TRUE, TRUE, 0);
-    gtk_widget_show (fontsel);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font size:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
-    font_size = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
-    pgram_defaults_dialog->font_size = GTK_SPIN_BUTTON(font_size);
-    gtk_box_pack_start(GTK_BOX (hbox), font_size, TRUE, TRUE, 0);
-    gtk_widget_show (font_size);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(pgram_defaults_dialog->show_background, 
-			       default_properties.show_background);
-  gtk_spin_button_set_value(pgram_defaults_dialog->shear_angle, 
-			    default_properties.shear_angle);
-
-  gtk_spin_button_set_value(pgram_defaults_dialog->padding,
-			    default_properties.padding);
-  attributes_get_default_font(&font, &font_height);
-  dia_font_selector_set_font(pgram_defaults_dialog->font, font);
-  gtk_spin_button_set_value(pgram_defaults_dialog->font_size, font_height);
-
-  return pgram_defaults_dialog->vbox;
-}
-
 static real
 pgram_distance_from(Pgram *pgram, Point *point)
 {
@@ -439,7 +257,7 @@ pgram_distance_from(Pgram *pgram, Point *point)
 
 static void
 pgram_select(Pgram *pgram, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   text_set_cursor(pgram->text, clicked_point, interactive_renderer);
   text_grab_focus(pgram->text, &pgram->element.object);
@@ -447,9 +265,10 @@ pgram_select(Pgram *pgram, Point *clicked_point,
   element_update_handles(&pgram->element);
 }
 
-static void
+static ObjectChange*
 pgram_move_handle(Pgram *pgram, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		  Point *to, ConnectionPoint *cp,
+		  HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -457,7 +276,7 @@ pgram_move_handle(Pgram *pgram, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&pgram->element, handle->id, to, reason);
+  element_move_handle(&pgram->element, handle->id, to, cp, reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -480,19 +299,24 @@ pgram_move_handle(Pgram *pgram, Handle *handle,
     break;
   }
   pgram_update_data(pgram, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 pgram_move(Pgram *pgram, Point *to)
 {
   pgram->element.corner = *to;
   
   pgram_update_data(pgram, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
-pgram_draw(Pgram *pgram, Renderer *renderer)
+pgram_draw(Pgram *pgram, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[4];
   Element *elem;
   real offs;
@@ -518,19 +342,19 @@ pgram_draw(Pgram *pgram, Renderer *renderer)
   }
 
   if (pgram->show_background) {
-    renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
-    renderer->ops->fill_polygon(renderer, 
+    renderer_ops->fill_polygon(renderer, 
 				pts, 4,
 				&pgram->inner_color);
   }
 
-  renderer->ops->set_linewidth(renderer, pgram->border_width);
-  renderer->ops->set_linestyle(renderer, pgram->line_style);
-  renderer->ops->set_dashlength(renderer, pgram->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, pgram->border_width);
+  renderer_ops->set_linestyle(renderer, pgram->line_style);
+  renderer_ops->set_dashlength(renderer, pgram->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_polygon(renderer, 
+  renderer_ops->draw_polygon(renderer, 
 			      pts, 4,
 			      &pgram->border_color);
 
@@ -548,6 +372,8 @@ pgram_update_data(Pgram *pgram, AnchorShape horiz, AnchorShape vert)
   Point p;
   real offs;
   real width, height;
+  real avail_width;
+  real top_left;
 
   /* save starting points */
   center = bottom_right = elem->corner;
@@ -558,14 +384,25 @@ pgram_update_data(Pgram *pgram, AnchorShape horiz, AnchorShape vert)
 
   /* this takes the shearing of the parallelogram into account, so the
    * text can be extend to the edges of the parallelogram */
+  text_calc_boundingbox(pgram->text, NULL);
   height = pgram->text->height * pgram->text->numlines + pgram->padding*2 +
     pgram->border_width;
   if (height > elem->height) elem->height = height;
 
+  avail_width = elem->width - (pgram->padding*2 + pgram->border_width +
+    fabs(pgram->shear_grad) * (elem->height + pgram->text->height
+			       * pgram->text->numlines));
+  if (avail_width < pgram->text->max_width) {
+    elem->width = (elem->width-avail_width) + pgram->text->max_width;
+    avail_width = pgram->text->max_width;
+  }
+
+  /*
   width = pgram->text->max_width + pgram->padding*2 + pgram->border_width +
     fabs(pgram->shear_grad) * (elem->height + pgram->text->height
 			       * pgram->text->numlines);
   if (width > elem->width) elem->width = width;
+  */
   
   /* move shape if necessary ... */
   switch (horiz) {
@@ -588,75 +425,92 @@ pgram_update_data(Pgram *pgram, AnchorShape horiz, AnchorShape vert)
   p = elem->corner;
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - pgram->text->height * pgram->text->numlines / 2 +
-    font_ascent(pgram->text->font, pgram->text->height);
+      pgram->text->ascent;
+  switch (pgram->text->alignment) {
+  case ALIGN_LEFT:
+    p.x -= avail_width/2;
+    break;
+  case ALIGN_RIGHT:
+    p.x += avail_width/2;
+    break;
+  case ALIGN_CENTER:
+    break;
+  }
   text_set_position(pgram->text, &p);
+  
+  /* 1/4 of how much more to the left the bottom line is */
+  offs = -(elem->height / 4.0 * pgram->shear_grad);
+  width = elem->width - 4.0*fabs(offs);
+  top_left = elem->corner.x;
+  if (offs < 0.0) {
+    top_left -= 4*offs;
+  }
 
   /* Update connections: */
-  pgram->connections[0].pos.x = elem->corner.x;
-  pgram->connections[0].pos.y = elem->corner.y;
-  pgram->connections[1].pos.x = elem->corner.x + elem->width / 4.0;
-  pgram->connections[1].pos.y = elem->corner.y;
-  pgram->connections[2].pos.x = elem->corner.x + elem->width / 2.0;
-  pgram->connections[2].pos.y = elem->corner.y;
-  pgram->connections[3].pos.x = elem->corner.x + elem->width * 3.0 / 4.0;
-  pgram->connections[3].pos.y = elem->corner.y;
-  pgram->connections[4].pos.x = elem->corner.x + elem->width;
-  pgram->connections[4].pos.y = elem->corner.y;
-  pgram->connections[5].pos.x = elem->corner.x;
-  pgram->connections[5].pos.y = elem->corner.y + elem->height / 4.0;
-  pgram->connections[6].pos.x = elem->corner.x + elem->width;
-  pgram->connections[6].pos.y = elem->corner.y + elem->height / 4.0;
-  pgram->connections[7].pos.x = elem->corner.x;
-  pgram->connections[7].pos.y = elem->corner.y + elem->height / 2.0;
-  pgram->connections[8].pos.x = elem->corner.x + elem->width;
-  pgram->connections[8].pos.y = elem->corner.y + elem->height / 2.0;
-  pgram->connections[9].pos.x = elem->corner.x;
-  pgram->connections[9].pos.y = elem->corner.y + elem->height * 3.0 / 4.0;
-  pgram->connections[10].pos.x = elem->corner.x + elem->width;
-  pgram->connections[10].pos.y = elem->corner.y + elem->height * 3.0 / 4.0;
-  pgram->connections[11].pos.x = elem->corner.x;
-  pgram->connections[11].pos.y = elem->corner.y + elem->height;
-  pgram->connections[12].pos.x = elem->corner.x + elem->width / 4.0;
-  pgram->connections[12].pos.y = elem->corner.y + elem->height;
-  pgram->connections[13].pos.x = elem->corner.x + elem->width / 2.0;
-  pgram->connections[13].pos.y = elem->corner.y + elem->height;
-  pgram->connections[14].pos.x = elem->corner.x + elem->width * 3.0 / 4.0;
-  pgram->connections[14].pos.y = elem->corner.y + elem->height;
-  pgram->connections[15].pos.x = elem->corner.x + elem->width;
-  pgram->connections[15].pos.y = elem->corner.y + elem->height;
-
-  offs = elem->height / 4.0 * pgram->shear_grad;
-  if (offs > 0) {
-    pgram->connections[0].pos.x += 4*offs;
-    pgram->connections[1].pos.x += 3*offs;
-    pgram->connections[2].pos.x += 2*offs;
-    pgram->connections[3].pos.x += offs;
-    pgram->connections[5].pos.x += 3*offs;
-    pgram->connections[6].pos.x -= offs;
-    pgram->connections[7].pos.x += 2*offs;
-    pgram->connections[8].pos.x -= 2*offs;
-    pgram->connections[9].pos.x += offs;
-    pgram->connections[10].pos.x -= 3*offs;
-    pgram->connections[12].pos.x -= offs;
-    pgram->connections[13].pos.x -= 2*offs;
-    pgram->connections[14].pos.x -= 3*offs;
-    pgram->connections[15].pos.x -= 4*offs;
-  } else {
-    pgram->connections[1].pos.x += offs;
-    pgram->connections[2].pos.x += 2*offs;
-    pgram->connections[3].pos.x += 3*offs;
-    pgram->connections[4].pos.x += 4*offs;
-    pgram->connections[5].pos.x -= offs;
-    pgram->connections[6].pos.x += 3*offs;
-    pgram->connections[7].pos.x -= 2*offs;
-    pgram->connections[8].pos.x += 2*offs;
-    pgram->connections[9].pos.x -= 3*offs;
-    pgram->connections[10].pos.x += offs;
-    pgram->connections[11].pos.x -= 4*offs;
-    pgram->connections[12].pos.x -= 3*offs;
-    pgram->connections[13].pos.x -= 2*offs;
-    pgram->connections[14].pos.x -= offs;
-  }
+  connpoint_update(&pgram->connections[0],
+		   top_left,
+		   elem->corner.y,
+		   DIR_NORTHWEST);
+  connpoint_update(&pgram->connections[1],
+		   top_left + width / 4.0,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&pgram->connections[2],
+		   top_left + width / 2.0,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&pgram->connections[3],
+		   top_left + width * 3.0 / 4.0,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&pgram->connections[4],
+		   top_left + width,
+		   elem->corner.y,
+		   DIR_NORTHEAST);
+  connpoint_update(&pgram->connections[5],
+		   top_left + offs,
+		   elem->corner.y + elem->height / 4.0,
+		   DIR_WEST);
+  connpoint_update(&pgram->connections[6],
+		   top_left + width + offs,
+		   elem->corner.y + elem->height / 4.0,
+		   DIR_EAST);
+  connpoint_update(&pgram->connections[7],
+		   top_left + 2.0 * offs,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST);
+  connpoint_update(&pgram->connections[8],
+		   top_left + width + 2.0 * offs,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST);
+  connpoint_update(&pgram->connections[9],
+		   top_left + 3.0 * offs,
+		   elem->corner.y + elem->height * 3.0 / 4.0,
+		   DIR_WEST);
+  connpoint_update(&pgram->connections[10],
+		   top_left + width + 3.0 * offs,
+		   elem->corner.y + elem->height * 3.0 / 4.0,
+		   DIR_EAST);
+  connpoint_update(&pgram->connections[11],
+		   top_left + 4.0 * offs,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTHWEST);
+  connpoint_update(&pgram->connections[12],
+		   top_left + 4.0 * offs + width / 4.0,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&pgram->connections[13],
+		   top_left + 4.0 * offs + width / 2.0,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&pgram->connections[14],
+		   top_left + 4.0 * offs + 3.0 * width / 4.0,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&pgram->connections[15],
+		   top_left + 4.0 * offs + width,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTHEAST);
 
   extra->border_trans = pgram->border_width/2.0;
   element_update_boundingbox(elem);
@@ -677,7 +531,7 @@ pgram_create(Point *startpoint,
   Object *obj;
   Point p;
   int i;
-  Font *font;
+  DiaFont *font = NULL;
   real font_height;
 
   init_default_values();
@@ -710,7 +564,9 @@ pgram_create(Point *startpoint,
   p.y += elem->height / 2.0 + font_height / 2;
   pgram->text = new_text("", font, font_height, &p, &pgram->border_color,
 			 ALIGN_CENTER);
-
+  text_get_attributes(pgram->text,&pgram->attrs);
+  dia_font_unref(font);
+  
   element_init(elem, 8, 16);
 
   for (i=0;i<16;i++) {
@@ -734,45 +590,6 @@ pgram_destroy(Pgram *pgram)
   element_destroy(&pgram->element);
 }
 
-static Object *
-pgram_copy(Pgram *pgram)
-{
-  int i;
-  Pgram *newpgram;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &pgram->element;
-  
-  newpgram = g_malloc0(sizeof(Pgram));
-  newelem = &newpgram->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-
-  newpgram->border_width = pgram->border_width;
-  newpgram->border_color = pgram->border_color;
-  newpgram->inner_color = pgram->inner_color;
-  newpgram->show_background = pgram->show_background;
-  newpgram->line_style = pgram->line_style;
-  newpgram->dashlength = pgram->dashlength;
-  newpgram->shear_angle = pgram->shear_angle;
-  newpgram->shear_grad = pgram->shear_grad;
-  newpgram->padding = pgram->padding;
-
-  newpgram->text = text_copy(pgram->text);
-  
-  for (i=0;i<16;i++) {
-    newobj->connections[i] = &newpgram->connections[i];
-    newpgram->connections[i].object = newobj;
-    newpgram->connections[i].connected = NULL;
-    newpgram->connections[i].pos = pgram->connections[i].pos;
-    newpgram->connections[i].last_pos = pgram->connections[i].last_pos;
-  }
-
-  return &newpgram->element.object;
-}
-
 static void
 pgram_save(Pgram *pgram, ObjectNode obj_node, const char *filename)
 {
